package com.example.wtf.controller;

import com.example.wtf.exception.ResourceForbidden;
import com.example.wtf.exception.ResourceNotFound;
import com.example.wtf.model.Image;
import com.example.wtf.model.User;
import com.example.wtf.repository.UserRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.linkTo;
import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.methodOn;

@RestController
@RequestMapping("/users")
public class UserController {
    @Autowired
    private UserRepository repository;

    @GetMapping("/get")
    public @ResponseBody CollectionModel<EntityModel<User>> getUsers() {
        List<EntityModel<User>> users = new ArrayList<>();

        for (User user : repository.findAll()) {
            EntityModel<User> userResource = EntityModel.of(
                    user,
                    linkTo(methodOn(UserController.class).getUser(user.getId())).withSelfRel(),
                    linkTo(methodOn(UserController.class).updateUser(user.getId(), new User())).withRel("Update user"),
                    linkTo(methodOn(UserController.class).deleteUser(user.getId())).withRel("Delete user")
            );
            users.add(userResource);
        }

        return CollectionModel.of(
                users,
                linkTo(methodOn(UserController.class).addUser(new User())).withRel("Add User")
        );
    }

    @GetMapping("/get/{id}")
    public @ResponseBody EntityModel<User> getUser(@PathVariable Long id) {
        User user = repository.findById(id)
                .orElseThrow(() -> new ResourceNotFound("User not found"));

        return EntityModel.of(
                user,
                linkTo(methodOn(UserController.class).updateUser(user.getId(), new User())).withRel("Update user"),
                linkTo(methodOn(UserController.class).deleteUser(user.getId())).withRel("Delete user"),
                linkTo(methodOn(UserController.class).getUsers()).withRel("Users")
        );
    }

    @PostMapping("/add")
    public @ResponseBody EntityModel<User> addUser(@RequestBody User user) {

        if (repository.existsByUsername(user.getUsername())) {
            throw new ResourceForbidden("Username forbidden");
        }
        if (repository.existsByEmail(user.getEmail())) {
            throw new ResourceForbidden("Email forbidden");
        }

        if (user.getImage() == null) {
            user.setImage(new Image("/default/path"));
        }
        if (user.getCreation() == null) {
            user.setCreation(LocalDate.now());
        }

        repository.save(user);

        return EntityModel.of(
                user,
                linkTo(methodOn(UserController.class).getUser(user.getId())).withSelfRel(),
                linkTo(methodOn(UserController.class).updateUser(user.getId(), new User())).withRel("Update user"),
                linkTo(methodOn(UserController.class).deleteUser(user.getId())).withRel("Delete user"),
                linkTo(methodOn(UserController.class).getUsers()).withRel("Users")
        );
    }

    @PutMapping("/update/{id}")
    public @ResponseBody EntityModel<User> updateUser(@PathVariable Long id,
                                                      @RequestBody User update) {
        User user = repository.findById(id)
                .orElseThrow(() -> new ResourceNotFound("User not found"));

        if (update.getUsername() != null && !update.getUsername().isEmpty()) {
            if (repository.existsByUsername(user.getUsername())) {
                throw new ResourceForbidden("Username forbidden");
            }
            user.setUsername(update.getUsername());
        }
        if (update.getEmail() != null && !update.getEmail().isEmpty()) {
            if (repository.existsByEmail(user.getEmail())) {
                throw new ResourceForbidden("Email forbidden");
            }
            user.setEmail(update.getEmail());
        }
        if (update.getPassword() != null && !update.getPassword().isEmpty()) {
            user.setPassword(update.getPassword());
        }
        if (update.getImage() != null) {
            user.setImage(update.getImage());
        }
        if (update.getCreation() != null) {
            user.setCreation(update.getCreation());
        }

        repository.save(user);

        return EntityModel.of(
                user,
                linkTo(methodOn(UserController.class).getUser(id)).withSelfRel(),
                linkTo(methodOn(UserController.class).deleteUser(id)).withRel("Delete user"),
                linkTo(methodOn(UserController.class).getUsers()).withRel("Users")
        );
    }

    @DeleteMapping("delete/{id}")
    public @ResponseBody EntityModel<User> deleteUser(@PathVariable Long id) {
        User user = repository.findById(id)
                .orElseThrow(() -> new ResourceNotFound("User not found"));

        repository.delete(user);

        return EntityModel.of(
                user,
                linkTo(methodOn(UserController.class).getUsers()).withRel("Users")
        );
    }
}